Rebol [
	Title: Name: "Re-Box!"
	Author: "Nenad Rakocevic"
	Email: dockimbel@free.fr
	Web: http://softinnov.org/rebol/rebox.shtml
	Date: 20/09/2001
	File: %rebox.r
	Version: 1.0.1
	Purpose: "Famous BoxWorld! game ported to Rebol"
	History: {
		1.0.1 20/09/2001 "Bugfix at line 287, add a remainder 100"
		1.0.0 12/07/2001 "First release!"
	}
]

ctx-rebox: context [

	tiles: [
		1	wall
		2	floor
		3	target
	]
	
	boxes: make block! 10
	map-img: make image! 480x420
	moves-file: %moves.ini
	maps: level: level-data: bg: madman: moves: best: best-list: f-goto: f-lev:
	go-lev: none

	load-bin: func [file][reduce bind load decompress read/binary file 'self]
	
	maps: load-bin %data1.bin
	imgs: load-bin %data2.bin

	if system/version/4 = 3 [font-fixed: "Fixedsys"]

	decode-tile: func [value [integer!]][
		any [select tiles value 'unknown]
	]

	tile-type?: func [pos [pair!]][
		pos: pos + 1x1
		to integer! pick pick level-data/map pos/y pos/x
	]

	box-at?: func [pos [pair!] /local box][
		foreach box boxes [if box/map-pos = pos [return box]]
		none
	]

	dir-to-pos: func [value [word!]][
		select [up 0x-1 down 0x1 left -1x0 right 1x0] value
	]

	add-moves: func [step][moves/text: to-string step + to-integer moves/text]

	update-hiscores: has [mv score][
		mv: to-integer moves/text
		either best-list [
			either score: pick best-list level [
				if mv < score [poke best-list level mv]
			][append best-list mv]
		][best-list: reduce [mv]]
		write moves-file mold best-list
	]

	check-win?: has [win? box][
		win?: yes
		foreach box boxes [win?: win? and box/good-pos]
		if win? [
			alert either level = 100 ["Congratulations! You have finished the game"]
				["You have done a good job!"]
			update-hiscores
			level: min 99 level + 1
			show-level
		]
	]

	bw-ss: stylize [
		bw-panel: panel 480x18 188.188.188 edge [size: 0]
		bw-text: tt 130x18 font [size: 14 name: font-fixed] para [origin: 2x0]
		bw-box: bw-text edge [effect: 'ibevel size: 1x1 color: 200.200.200]

		bw-button: bw-text 50x15 center edge [color: 200.200.200 effect: 'bevel]
			para []
			font [name: font-sans-serif size: 11]
			feel [
				redraw: func [face act pos][
					face/edge/size: pick [1x1 none] face/state
					face/para/scroll: pick [0x-1 0x0] face/state
				]
				over: func [face over? offset][
					face/state: over?
					show face
				]
				engage: func [face action event][
					switch action [
						down [face/edge/effect: 'ibevel]
						up [
							face/edge/effect: 'bevel
							if face/state [do-face face none]
						]
						away [face/state: off]
						over [face/state: on]
					]
					show face
				]
			]
			with [state: off init: copy []]

		sprite: face 30x30 with [
			name: 'box
			effect:
			effects:
			undo-pos: none
			good-pos: no
			map-pos: 0x0	; counted in blocks
			init: copy []
			locate: func [pos [pair!]][
				map-pos: :pos
				offset: to-pair reduce [pos/x * 30 pos/y * 30 + 16]
				if name = 'box [
					good-pos: 'floor <> decode-tile tile-type? pos
					effect: pick effects good-pos
				]
			]
			can-move?: func [value [word!] /local new][
				new: tile-type? map-pos + dir-to-pos value
				find [2 3] new
			]
		]

		man: sprite rate 2 feel [
			engage: func [face action event][
				if action = 'time [
					face/mad-state: not face/mad-state
					face/image: pick face/images face/mad-state
					show face
				]
			]
		] with [
			name: 'man
			images: image: mad-state: undo-box: none
			turn: func [value [word!]][
				image: first images: imgs/:value
				mad-state: on
			]
			undo-move: does [
				if all [found? undo-pos undo-pos <> map-pos][
					locate undo-pos
					undo-box/locate undo-box/undo-pos
					add-moves -1
					show box-world
				]
			]
			want-move: func [value [word!] /local box c-pos b-pos][
				c-pos: map-pos + dir-to-pos value
				turn value
				if can-move? value [
					box: box-at? c-pos
					either not box [
						locate c-pos
						show self
					][
						b-pos: box/map-pos + dir-to-pos value
						if all [box/can-move? value	not box-at? b-pos][
							undo-pos: map-pos
							locate c-pos
							undo-box: :box
							box/undo-pos: box/map-pos
							box/locate b-pos
							add-moves 1
							show reduce [self box moves]
							check-win?
						]
					]
				]
			]	
		]
	]

	for-pair: func ['word start [pair!] end [pair!] body [block!] /local op do-body val][
		do-body: func reduce [[throw] word] body
		op: :lesser-or-equal?
		val: start
		while [op val/y end/y][
			val/x: start/x
			while [op val/x end/x][do-body val val/x: val/x + 1]
			val/y: val/y + 1
		]
	]

	draw-map: has [tile][
		level-data: maps/:level
		for-pair pos 0x0 15x13 [
			if not zero? tile: tile-type? pos [
				tile: select imgs decode-tile tile
				for line 0 29 1 [
					change at bg/image line * 480 + (pos/x * 30) + (pos/y * 14400) + 1
						copy/part at tile line * 30 + 1 120
				]
			]
		]
	]

	draw-sprites: has [bx pos][
		foreach pos level-data/boxes [
			append box-world/pane bx: make bw-ss/sprite [
				image: imgs/box
				effects: [[colorize 255.0.0] none]
				locate pos
			]
			append boxes bx
		]
		madman/locate level-data/start
	]

	goto-lay: layout [
		size 250x100
		backdrop 200.200.200
		across tt font-name font-fixed "Input new stage: "
		f-goto: field bold 60 return
		pad 30x10 button 60 cr "Ok" [
			go-lev: load f-goto/text 
			if all [best-list integer? go-lev positive? go-lev][
				level: min go-lev 1 + length? best-list
				show-level
			]
			unview
		]
		button 60 "Cancel" [unview]
		do [focus f-goto]
	]

	credits-lay: layout [
		size 378x350
		style ctxt text bold center 378x20 
		backdrop white
		at 0x0 image imgs/credits below
		ctxt "Original game by Jeng-Long Jiang"
		pad 0x-10 ctxt "Rebol port done by Nenad Rakocevic"
		pad 0x-10 ctxt "(dockimbel@free.fr - 12/07/2001)"
		pad 150x-8 button 60x20 cr "ok" [unview]
	]

	box-world: layout [
		styles bw-ss
		size 480x455
		at 0x0 bw-panel [
			origin 0x0 across
			bw-button "Undo" #"^Z"	[madman/undo-move]
			bw-button "Retry" #"^R"	[init-sprites show box-world]
			bw-button "Goto..."		[view/new center-face goto-lay]
			bw-button "About..." 	[view/new center-face credits-lay]
		]
		at 0x16 bg: image map-img effect []
		at 0x435 bw-panel 480x22 [
			at 10x1 bw-box "Your moves:    "
			at 75x2 moves: bw-text right 60 "0"
			at 150x1 bw-box "Best moves:    "
			at 215x2 best: bw-text right 60 "0"
			at 290x1 bw-box 90 "Level:    "
			at 315x1 f-lev: bw-text right 60 "0"
		]
		madman: man keycode [left right up down] [madman/want-move value]
	]

	init-sprites: does [
		forall boxes [remove find box-world/pane boxes/1]
		clear boxes
		recycle
		draw-sprites
		madman/turn 'down
		madman/undo-pos: none
		moves/text: "0"
		best/text: any [all [best-list pick best-list level] "N/A"]
		f-lev/text: mold level
	]

	show-level: does [
		recycle
		bg/image: make image! 480x420
		draw-map
		init-sprites
		show box-world
	]

	set 'start-rebox does [
		if exists? moves-file [
			best-list: load moves-file
			level: remainder 1 + length? best-list 100
		]
		draw-map
		init-sprites
		view/title center-face box-world "Re-Box!"
	]
]

start-rebox